using System;
using System.Net;
using System.IO;
using System.Data;

using System.Text;
using System.Net.Mail;

namespace Gateways
{
    public class OrderEmail : GatewayBase
    {
        public OrderEmail()
        {
        }

        private string Clean(object o)
        {
            if (o == null || o is DBNull)
                return "";
            else
                return o.ToString();
        }

        public override GatewayResult InternalAuthorize(int OrderID, string xFirstName, string xLastName, string xAddress1, string xAddress2, string xCity, string xState, string xZip, string xCountry, string xPhone, string xCustomerID, string xCustomerIP, decimal xAmount, string CardNumber, string ExpDate, string CSC)
        {
            int ShippingAddressID, BillingAddressID;
            string firstname, lastname, address, city, state, zip, phone, customerid, customerip, amount, tax, shipping, shippingmethod, ponumber, Comments;

            DataRow drOrder = Database.Orders_GetByOrderID(OrderID);
            ShippingAddressID = (int)drOrder["ShippingAddressID"];
            BillingAddressID = (int)drOrder["BillingAddressID"];
            customerid = drOrder["CustomerID"].ToString();
            customerip = (string)drOrder["IPAddress"];
            tax = String.Format("{0:F}", drOrder["TaxAmount"]);
            shipping = String.Format("{0:F}", drOrder["ShippingAmount"]);
            shippingmethod = Clean(drOrder["ShippingMethod"]);
            amount = String.Format("{0:F}", drOrder["Amount"]);
            ponumber = Clean(drOrder["PONumber"]);
            Comments = (string)drOrder["UserComments"];

            DataRow drBilling = Database.OrderAddresses_GetByOrderAddressID(BillingAddressID);
            firstname = (string)drBilling["FirstName"];
            lastname = (string)drBilling["LastName"];
            address = (string)drBilling["Address"];
            city = (string)drBilling["City"];
            state = (string)drBilling["State"];
            zip = (string)drBilling["ZipCode"];
            phone = (string)drBilling["Phone"];

            DataRow drShipping = Database.OrderAddresses_GetByOrderAddressID(ShippingAddressID);

            DataTable dtOrderItems = Database.OrderItems_GetForAdmin(OrderID);

            DataTable dtDiscounts = Database.OrderDiscounts_GetByOrderID(OrderID);

            // build the email string
            StringBuilder sb = new StringBuilder();
            sb.Append("<head><style> body, td { font-size: 8pt; font-family: Arial; } </style></head>");

            sb.Append("Order ID: <b>");
            sb.Append(OrderID.ToString());
            sb.Append("</b><br>");

            sb.Append("<b>Billing Information</b>");
            sb.Append("<table>");
            sb.Append("<tr><td>First Name:</td><td>");
            sb.Append(firstname);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Last Name:</td><td>");
            sb.Append(lastname);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Address:</td><td>");
            sb.Append(address);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>City:</td><td>");
            sb.Append(city);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>State:</td><td>");
            sb.Append(state);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Zip:</td><td>");
            sb.Append(zip);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Phone:</td><td>");
            sb.Append(phone);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>PO Number:</td><td>");
            sb.Append(ponumber);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Card Number:</td><td>");
            sb.Append(CardNumber);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Expiration Date:</td><td>");
            sb.Append(ExpDate);
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Security Code:</td><td>");
            sb.Append(CSC);
            sb.Append("</td></tr>");
            sb.Append("</table><br>");

            sb.Append("<b>Shipping Information</b>");
            sb.Append("<table>");
            sb.Append("<tr><td>First Name:</td><td>");
            sb.Append(Clean(drShipping["FirstName"]));
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Last Name:</td><td>");
            sb.Append(Clean(drShipping["LastName"]));
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Address:</td><td>");
            sb.Append(Clean(drShipping["Address"]));
            sb.Append("</td></tr>");
            sb.Append("<tr><td>City:</td><td>");
            sb.Append(Clean(drShipping["City"]));
            sb.Append("</td></tr>");
            sb.Append("<tr><td>State:</td><td>");
            sb.Append(Clean(drShipping["State"]));
            sb.Append("</td></tr>");
            sb.Append("<tr><td>Zip Code:</td><td>");
            sb.Append(Clean(drShipping["ZipCode"]));
            sb.Append("</td><tr>");
            sb.Append("<tr><td>Phone:</td><td>");
            sb.Append(Clean(drShipping["Phone"]));
            sb.Append("</td><tr>");
            sb.Append("<tr><td>Email Address:</td><td><a href='mailto:");
            sb.Append(Clean(drShipping["EmailAddress"]));
            sb.Append("'>");
            sb.Append(Clean(drShipping["EmailAddress"]));
            sb.Append("</a></td></tr>");
            sb.Append("</table><br>");

            sb.Append("<b>Additional Information</b>");
            sb.Append("<tr><td>Special Instructions/Comments:</td><td>");
            sb.Append(Comments);
            sb.Append("</td></tr>");
            sb.Append("</table><br>");

            sb.Append("<b>Order Contents</b>");
            sb.Append("<table style='width: 500px'><tr><th>Quantity</th><th>Product Name</th><th>Item Price</th><th>Price</th></tr>");

            bool Alternator = true;
            foreach (DataRow dr in dtOrderItems.Rows)
            {
                Alternator = !Alternator;

                sb.Append("<tr valign='top' style='background-color: ");
                sb.Append((Alternator ? "#FFFFFF" : "#DEDEDE"));
                sb.Append("'><td>");
                sb.Append((int)dr["Quantity"]);
                sb.Append("</td><td>");
                sb.Append((string)dr["ItemName"]);
                sb.Append("(");
                sb.Append((string)dr["SKU"]);
                sb.Append(")</td><td>");
                sb.Append(String.Format("{0:f}", (decimal)dr["Price"]));
                sb.Append("</td><td>");
                sb.Append(String.Format("{0:f}", (decimal)dr["Cost"]));
                sb.Append("</td></tr>");
            }
            foreach (DataRow dr in dtDiscounts.Rows)
            {
                sb.Append("<tr><td colspan='2'></td><td>Discount(");
                sb.Append((string)dr["DiscountName"]);
                sb.Append("):</td><td>-");
                sb.Append(String.Format("{0:C}", (decimal)dr["DiscountAmount"]));
                sb.Append("</td></tr>");
            }
            sb.Append("<tr><td colspan='2'></td><td>Tax:</td><td>");
            sb.Append(tax);
            sb.Append("</td></tr>");
            sb.Append("<tr><td colspan='2'></td><td>Shipping (");
            sb.Append(shippingmethod);
            sb.Append("):</td><td>");
            sb.Append(shipping);
            sb.Append("</td></tr>");
            sb.Append("<tr><td colspan='2'></td><td>Total:</td><td>");
            sb.Append(amount);
            sb.Append("</td></tr>");
            sb.Append("</table>");

            // grab the email address that this will be sent to
            string recipient = Configuration.Get("Gateways.OrderEmail.RecipientAddress");

            GatewayResult gwResult = new GatewayResult();
            gwResult.Operation = GatewayOperation.Authorize;
            gwResult.PaymentReferenceCode = "------";
            gwResult.ResponseCode = "";
            gwResult.ResponseText = "";

            // send off email
            try
            {
                MailMessage msg = new MailMessage();
                msg.From = new MailAddress("no-reply@webadvanced.com", "Web Orders");
                msg.To.Add(recipient);
                msg.Subject = "A new order has been placed";
                msg.IsBodyHtml = true;
                msg.Body = sb.ToString();

                SmtpClient sc = new SmtpClient();
                sc.Send(msg);

                gwResult.Operation = GatewayOperation.Authorize;
                gwResult.PaymentReferenceCode = "0";
                gwResult.ResponseCode = "1";
                gwResult.ResponseText = "OrderEmail used - no authorization needed";
                gwResult.Successful = true;
            }
            catch
            {
                gwResult.PaymentReferenceCode = "0";
                gwResult.ResponseCode = "1";
                gwResult.ResponseText = "An error occurred while attempting to send the order email.";
                gwResult.Successful = false;
            }

            return gwResult;
        }
        public override GatewayResult InternalCapture(string PaymentReferenceCode, decimal Amount)
        {
            GatewayResult result = new GatewayResult();
            result.Operation = GatewayOperation.Authorize;
            result.Successful = true;
            result.ResponseCode = "";
            result.ResponseText = "";
            result.PaymentReferenceCode = "------";
            return result;
        }
        public override GatewayResult InternalVoid(string PaymentReferenceCode)
        {
            GatewayResult result = new GatewayResult();
            result.Operation = GatewayOperation.Authorize;
            result.Successful = true;
            result.ResponseCode = "";
            result.ResponseText = "";
            result.PaymentReferenceCode = "------";
            return result;
        }
        public override GatewayResult InternalCredit(string PaymentReferenceCode, string LastFour, decimal Amount)
        {
            GatewayResult result = new GatewayResult();
            result.Operation = GatewayOperation.Authorize;
            result.Successful = true;
            result.ResponseCode = "";
            result.ResponseText = "";
            result.PaymentReferenceCode = "------";
            return result;
        }
    }
}
